Package: treespace
Title: Statistical Exploration of Landscapes of Phylogenetic Trees
Date: 2025-08-26
Version: 1.1.4.4
Authors@R: c(TJ = person("Thibaut", "Jombart", email="thibautjombart@gmail.com", role = "aut", comment = c(ORCID = "0000-0003-2226-8692")),
 	     MK = person("Michelle", "Kendall", email="michelle.kendall@ndm.ox.ac.uk", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-7344-7071")),
	     JAG = person("Jacob", "Almagro-Garcia", role = "aut", comment = c(ORCID = "0000-0002-0595-7333")),
	     CC = person("Caroline", "Colijn", role = "aut", comment = c(ORCID = "0000-0001-6097-6708")))
Description: Tools for the exploration of distributions of phylogenetic trees.
    This package includes a 'shiny' interface which can be started from R using
    treespaceServer(). 
    For further details see Jombart et al. (2017) <DOI:10.1111/1755-0998.12676>.
Depends: R (>= 3.4.0), ape, ade4
Imports: adegenet, adegraphics, combinat, compiler, distory, fields,
        htmlwidgets, MASS, methods, parallel, phangorn, phytools, Rcpp,
        rgl, RLumShiny, scatterD3, shiny, shinyBS, utils
LinkingTo: Rcpp
Suggests: ggplot2, igraph, knitr, pander, RColorBrewer, reshape2,
        rmarkdown, sf, testthat
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Collate: RcppExports.R metrics.R medTree.R treespace.R findGroves.R
        plotGroves.R plotTreeDiff.R servers.R transmissionTrees.R
        data.R makeCollapsedTree.R relatedTreeDist.R simulateIndTree.R
        tipsMRCAdepths.R treeConcordance.R
RoxygenNote: 7.3.2
VignetteBuilder: knitr
URL: https://cran.r-project.org/package=treespace,
        https://github.com/thibautjombart/treespace
BugReports: https://github.com/thibautjombart/treespace/issues
NeedsCompilation: yes
Packaged: 2025-08-27 13:55:25 UTC; michellekendall
Repository: CRAN
Date/Publication: 2025-08-27 16:30:02 UTC
Author: Thibaut Jombart [aut] (ORCID: <https://orcid.org/0000-0003-2226-8692>),
  Michelle Kendall [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7344-7071>),
  Jacob Almagro-Garcia [aut] (ORCID:
    <https://orcid.org/0000-0002-0595-7333>),
  Caroline Colijn [aut] (ORCID: <https://orcid.org/0000-0001-6097-6708>)
Maintainer: Michelle Kendall <michelle.kendall@ndm.ox.ac.uk>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 13:51:01 UTC; unix
Archs: treespace.so.dSYM
