# Generated by roxygen2: do not edit by hand

export(.render.server.info)
export(findGroves)
export(findMRCIs)
export(linearMrca)
export(makeCollapsedTree)
export(medTree)
export(multiDist)
export(plotGroves)
export(plotGrovesD3)
export(plotTreeDiff)
export(refTreeDist)
export(relatedTreeDist)
export(simulateIndTree)
export(tipDiff)
export(tipsMRCAdepths)
export(treeConcordance)
export(treeDist)
export(treeVec)
export(treespace)
export(treespaceServer)
export(wiwMedTree)
export(wiwTreeDist)
import(ape)
import(rgl)
import(scatterD3)
import(shiny)
importFrom(MASS,Shepard)
importFrom(RLumShiny,jscolorInput)
importFrom(Rcpp,evalCpp)
importFrom(ade4,cailliez)
importFrom(ade4,dudi.pco)
importFrom(ade4,is.euclid)
importFrom(adegenet,.readExt)
importFrom(adegenet,bluepal)
importFrom(adegenet,funky)
importFrom(adegenet,lightseasun)
importFrom(adegenet,num2col)
importFrom(adegenet,transp)
importFrom(adegraphics,insert)
importFrom(adegraphics,s.class)
importFrom(adegraphics,s.label)
importFrom(adegraphics,s1d.barchart)
importFrom(combinat,combn)
importFrom(combinat,combn2)
importFrom(compiler,cmpfun)
importFrom(distory,dist.multiPhylo)
importFrom(fields,rdist)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,layout)
importFrom(graphics,plot)
importFrom(htmlwidgets,saveWidget)
importFrom(methods,is)
importFrom(parallel,mcmapply)
importFrom(phangorn,Children)
importFrom(phangorn,Descendants)
importFrom(phangorn,KF.dist)
importFrom(phangorn,RF.dist)
importFrom(phangorn,path.dist)
importFrom(phangorn,wRF.dist)
importFrom(phytools,paste.tree)
importFrom(shinyBS,bsTooltip)
importFrom(stats,as.dist)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(utils,packageDescription)
useDynLib(treespace)
