---
title: "treespace worked example: Transmission trees"
author: "Michelle Kendall"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{treespace worked example: Transmission trees}
  \usepackage[utf8]{inputenc}
---
  

[This vignette](https://thibautjombart.github.io/treespace/articles/TransmissionTreesVignette.html) demonstrates the use of *treespace* to compare a collection of transmission trees as proposed in [Kendall et al. 2018](https://projecteuclid.org/journals/statistical-science/volume-33/issue-1/Estimating-Transmission-from-Genetic-and-Epidemiological-Data--A-Metric/10.1214/17-STS637.full).